using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using PhraseAnalyzer;
using GrammerAnalyzer;
using Calculator;

namespace CalUI
{
    public partial class MainForm : Form
    {
        private PhraseAnalyzer.PhraseStorage ps;
        private GrammerAnalyzer.SemanticAnalyzer sa;
        private Calculator.CalUnit cunit;
        private DisplayFormat displayFormat = DisplayFormat.Auto;
        private PhraseAnalyzer.PhraseAnalyzer pa;
        private Calculator.CalculationMode cm = CalculationMode.Degree;

        public MainForm()
        {
            InitializeComponent();
        }

        private void btnEqual_Click(object sender, System.EventArgs e)
        {
            tbTip.Text = String.Empty;
            //Ϊ
            if (tbExpression.Text == String.Empty)
                return;
            //ûд#
            if (tbExpression.Text.IndexOf('#') != -1)
            {
                tbTip.Text = "ʽдڷǷ";
                return;
            }
            ps = new PhraseAnalyzer.PhraseStorage();
            //ʷ
            pa = new PhraseAnalyzer.PhraseAnalyzer(tbExpression.Text + "#=", ref ps);
            if (pa.Succeed == false)
            {
                tbTip.Text = "ʽдڷǷ";
                return;
            }
            //ķ
            sa = new GrammerAnalyzer.SemanticAnalyzer(ref ps);
            if (sa.Check() == false)
            {	//ķ
                tbTip.Text = sa.ErrorTip;
                return;
            }
            //ʾ滻NXıʽ
            string tiptext = tbExpression.Text.ToUpper();
            tiptext = tiptext.Replace("AX", ConstantTable.AX.ToString());
            tiptext = tiptext.Replace("BX", ConstantTable.BX.ToString());
            tiptext = tiptext.Replace("CX", ConstantTable.CX.ToString());
            tiptext = tiptext.Replace("DX", ConstantTable.DX.ToString());
            tiptext = tiptext.Replace("EX", ConstantTable.EX.ToString());
            tiptext = tiptext.Replace("FX", ConstantTable.FX.ToString());
            tiptext = tiptext.Replace("ANS", ConstantTable.ANS.ToString());
            tbTip.Text = tiptext;
            //ʷȷ
            cunit = new CalUnit(ref ps);
            cunit.Mode = cm;
            cunit.Run();
            //ʾ
            DisplayResult();

            ps = null;
            pa = null;
            sa = null;
            cunit = null;
            GC.Collect();
        }

        private void DisplayResult()
        {
            switch (displayFormat)
            {
                case DisplayFormat.Auto:
                    tbResult.Text = ConstantTable.ANS.ToString();
                    break;
                case DisplayFormat.Fixed:
                    tbResult.Text = ConstantTable.ANS.ToString("0.00");
                    break;
                case DisplayFormat.Grouped:
                    tbResult.Text = ConstantTable.ANS.ToString("n");
                    break;
                case DisplayFormat.Scientific:
                    tbResult.Text = ConstantTable.ANS.ToString("E");
                    break;
            }
        }

        private void btnPI_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "PI";
        }

        private void btnLeftBracket_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "(";
        }

        private void btnRightBracket_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += ")";
        }

        private void btnPlus_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "+";
        }

        private void btnMinus_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "-";
        }

        private void btnTimes_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "*";
        }

        private void btnDivide_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "/";
        }

        private void btnMod_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "%";
        }

        private void btnPower_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "^";
        }

        private void btnNum1_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "1";
        }

        private void btnNum2_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "2";
        }

        private void btnNum3_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "3";
        }

        private void btnNum4_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "4";
        }

        private void btnNum5_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "5";
        }

        private void btnNum6_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "6";
        }

        private void btnNum7_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "7";
        }

        private void btnNum8_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "8";
        }

        private void btnNum9_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "9";
        }

        private void btnNum0_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "0";
        }

        private void btnDot_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += ".";
        }

        private void btnExp_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "e^";
        }

        private void btnSin_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "sin";
        }

        private void btnCos_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "cos";
        }

        private void btnTg_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "tg";
        }

        private void btnCtg_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "ctg";
        }

        private void btnAsin_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "asin";
        }

        private void btnAcos_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "acos";
        }

        private void btnAtg_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "atg";
        }

        private void btnActg_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "actg";
        }

        private void btnSbrt_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "sbrt";
        }

        private void btnCbrt_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "cbrt";
        }

        private void btnLn_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "ln";
        }

        private void btnLog_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "log";
        }

        private void btnLg_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "lg";
        }

        private void button1_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "1/x";
        }

        private void button2_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "^2";
        }

        private void button3_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "^3";
        }

        private void button4_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "!";
        }

        /// <summary>
        /// ÿؼǷ
        /// </summary>
        /// <param name="enable"></param>
        private void SetEnable(bool enable)
        {
            btnLeftBracket.Enabled = enable;
            btnRightBracket.Enabled = enable;
            btnNum1.Enabled = enable;
            btnNum2.Enabled = enable;
            btnNum3.Enabled = enable;
            btnNum4.Enabled = enable;
            btnNum5.Enabled = enable;
            btnNum6.Enabled = enable;
            btnNum7.Enabled = enable;
            btnNum8.Enabled = enable;
            btnNum9.Enabled = enable;
            btnNum0.Enabled = enable;
            btnDot.Enabled = enable;
            btnLn.Enabled = enable;
            btnLog.Enabled = enable;
            btnLg.Enabled = enable;
            btnX_1.Enabled = enable;
            btnX_2.Enabled = enable;
            btnX_3.Enabled = enable;
            btnFact.Enabled = enable;
            btnSbrt.Enabled = enable;
            btnCbrt.Enabled = enable;
            btnSin.Enabled = enable;
            btnCos.Enabled = enable;
            btnTg.Enabled = enable;
            btnCtg.Enabled = enable;
            btnAsin.Enabled = enable;
            btnAcos.Enabled = enable;
            btnAtg.Enabled = enable;
            btnActg.Enabled = enable;
            btnANS.Enabled = enable;
            btnPI.Enabled = enable;
            btnExp.Enabled = enable;
            btnPlus.Enabled = enable;
            btnMinus.Enabled = enable;
            btnTimes.Enabled = enable;
            btnDivide.Enabled = enable;
            btnMod.Enabled = enable;
            btnPower.Enabled = enable;
            btnEqual.Enabled = enable;
            btnSto.Enabled = enable;
        }


        private bool isStoClicked = false;
        private void btnSto_Click(object sender, System.EventArgs e)
        {
            this.SetEnable(false);
            isStoClicked = true;
            //¼
            btnEqual_Click(btnEqual, null);
        }

        private void btnANS_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "ANS";
        }

        private void btnAx_Click(object sender, System.EventArgs e)
        {
            if (isStoClicked == true)
            {
                ConstantTable.AX = ConstantTable.ANS;
                tbTip.Text = "AX = " + ConstantTable.ANS.ToString();
                this.SetEnable(true);
                isStoClicked = false;
            }
            else
                tbExpression.Text += "AX";
        }

        private void btnBx_Click(object sender, System.EventArgs e)
        {
            if (isStoClicked == true)
            {
                ConstantTable.BX = ConstantTable.ANS;
                tbTip.Text = "BX = " + ConstantTable.ANS.ToString();
                this.SetEnable(true);
                isStoClicked = false;
            }
            else
                tbExpression.Text += "BX";
        }

        private void btnCx_Click(object sender, System.EventArgs e)
        {
            if (isStoClicked == true)
            {
                ConstantTable.CX = ConstantTable.ANS;
                tbTip.Text = "CX = " + ConstantTable.ANS.ToString();
                this.SetEnable(true);
                isStoClicked = false;
            }
            else
                tbExpression.Text += "CX";
        }

        private void btnDx_Click(object sender, System.EventArgs e)
        {
            if (isStoClicked == true)
            {
                ConstantTable.DX = ConstantTable.ANS;
                tbTip.Text = "DX = " + ConstantTable.ANS.ToString();
                this.SetEnable(true);
                isStoClicked = false;
            }
            else
                tbExpression.Text += "DX";
        }

        private void btnEx_Click(object sender, System.EventArgs e)
        {
            if (isStoClicked == true)
            {
                ConstantTable.EX = ConstantTable.ANS;
                tbTip.Text = "EX = " + ConstantTable.ANS.ToString();
                this.SetEnable(true);
                isStoClicked = false;
            }
            else
                tbExpression.Text += "EX";
        }

        private void btnAC_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text = String.Empty;
            tbTip.Text = String.Empty;
            tbResult.Text = "0.0";
            ConstantTable.ANS = 0;
        }

        private void btnClr_Click(object sender, System.EventArgs e)
        {
            ConstantTable.ClearAll();
            tbTip.Text = "б";
        }

        private void btnNegative_Click(object sender, System.EventArgs e)
        {
            tbExpression.Text += "-";
        }

        private void tbExpression_KeyPress(object sender, System.Windows.Forms.KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Escape)
            {
                tbExpression.Text = "";
                e.Handled = true;
            }
            else if (e.KeyChar == (char)Keys.Enter)
            {
                //Enter¼
                btnEqual_Click(btnEqual, null);
                e.Handled = true;
            }
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AboutForm af = new AboutForm();
            af.ShowDialog();
        }

        private void variableToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
        {
            aXToolStripMenuItem.Text = "AX: " + ConstantTable.AX.ToString();
            bXToolStripMenuItem.Text = "BX: " + ConstantTable.BX.ToString();
            cXToolStripMenuItem.Text = "CX: " + ConstantTable.CX.ToString();
            dXToolStripMenuItem.Text = "DX: " + ConstantTable.DX.ToString();
            eXToolStripMenuItem.Text = "EX: " + ConstantTable.EX.ToString();
            fXToolStripMenuItem.Text = "FX: " + ConstantTable.FX.ToString();
            aNSToolStripMenuItem.Text = "ANS: " + ConstantTable.ANS.ToString();
        }


        private void MainForm_Load(object sender, EventArgs e)
        {

        }

        private void fixedDecimalToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (!fixedDecimalToolStripMenuItem.Checked)
            {
                //set ui
                autoToolStripMenuItem.Checked = false;
                fixedDecimalToolStripMenuItem.Checked = true;
                scientificToolStripMenuItem.Checked = false;
                groupedDigitToolStripMenuItem.Checked = false;
                //set value
                displayFormat = DisplayFormat.Fixed;

                DisplayResult();
            }
        }


        private void scientificToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (!scientificToolStripMenuItem.Checked)
            {
                //set ui
                autoToolStripMenuItem.Checked = false;
                fixedDecimalToolStripMenuItem.Checked = false;
                scientificToolStripMenuItem.Checked = true;
                groupedDigitToolStripMenuItem.Checked = false;
                //set value
                displayFormat = DisplayFormat.Scientific;

                DisplayResult();
            }
        }

        private void autoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (!autoToolStripMenuItem.Checked)
            {
                //set ui
                autoToolStripMenuItem.Checked = true;
                fixedDecimalToolStripMenuItem.Checked = false;
                scientificToolStripMenuItem.Checked = false;
                groupedDigitToolStripMenuItem.Checked = false;
                //set value
                displayFormat = DisplayFormat.Auto;

                DisplayResult();
            }
        }

        private void groupedDigitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (!groupedDigitToolStripMenuItem.Checked)
            {
                //set ui
                autoToolStripMenuItem.Checked = false;
                fixedDecimalToolStripMenuItem.Checked = false;
                scientificToolStripMenuItem.Checked = false;
                groupedDigitToolStripMenuItem.Checked = true;
                //set value
                displayFormat = DisplayFormat.Grouped;

                DisplayResult();
            }
        }

        private void miniToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (!miniToolStripMenuItem.Checked)
            {
                fullToolStripMenuItem.Checked = false;
                miniToolStripMenuItem.Checked = true;
                this.Height = 160;
            }
        }

        private void fullToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (!fullToolStripMenuItem.Checked)
            {
                fullToolStripMenuItem.Checked = true;
                miniToolStripMenuItem.Checked = false;
                this.Height = 397;
            }
        }

        private void degreeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (!degreeToolStripMenuItem.Checked)
            {
                degreeToolStripMenuItem.Checked = true;
                radiusToolStripMenuItem.Checked = false;
                cm = CalculationMode.Degree;
            }
        }

        private void radiusToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (!radiusToolStripMenuItem.Checked)
            {
                degreeToolStripMenuItem.Checked = false;
                radiusToolStripMenuItem.Checked = true;
                cm = CalculationMode.Radius;
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }
    }
}