#region BluePrint.Calculator, Copyright 2005 BluePrint Work Group, Programmed by Tony Qu
/* ************************************************************
 Ŀƣѧ
 ʵԣC#
 ߣTony Qu
 ߲ͣhttp://www.cnblogs.com/tonyqus
 ģƣ㵥Ԫ
 ģռ䣺Calculator
 ǰļϢCalculator.CalUnit㵥Ԫ
 Ŀǰ汾1.0.1.2
 
 ǰļ޶  
	Who			Date			Version			Comment
	Tony Qu		2005.5.5		1.0.0.0			
	Tony Qu		2005.5.10		1.0.1.0			㷨ȼ
	Tony Qu		2005.5.23		1.0.1.2			ܼ(3)-5
												cbrt㹫ʽ
 * Tony Qu      2006.8.12       1.0.1.3         ӶDegree/Radiusл
*************************************************************/
#endregion  

using System;
using System.Collections;
using PhraseAnalyzer;
using GrammerAnalyzer;

namespace Calculator
{
	/// <summary>
	/// Calculation Unit
	/// </summary>
	public class CalUnit:IDisposable
	{
		private Stack _optr=null;	//Operator Stack
		private Stack _opnd=null;	//Number Stack
		private PhraseStorage _ps=null;
        private CalculationMode _cm = CalculationMode.Degree;

		public CalUnit(ref PhraseStorage ps)
		{
			_optr=new Stack();
			_opnd=new Stack();
			_ps=ps;
		}
        ~CalUnit()
        {
            _optr = null;
            _opnd = null;
            _ps = null;
        }
        public void Dispose()
        {
            _optr = null;
            _opnd = null;
            _ps = null;
        }
        /// <summary>
        /// Calculation Mode: Degree/Radius
        /// </summary>
        public CalculationMode Mode
        {
            get { return _cm; }
            set { _cm = value; }
        }
		/// <summary>
		/// Factorial
		/// </summary>
		/// <param name="i">Floor</param>
		/// <returns></returns>
		private double Factorial(double i) 
		{ 
			return((i <= 1) ? 1 : (i * Factorial(i-1))); 
		}
		/// <summary>
		/// Calculate by phrase type
		/// </summary>
		/// <param name="a">Operator 1</param>
		/// <param name="b">Operator 2</param>
		/// <param name="pt">Phrase Type</param>
		/// <returns>Result</returns>
		private double Calculate(PhraseType pt)
		{
			//get the operators
			double a,b;
			a=b=0.0;
			OperandType ot=Operator.OperandCount(pt);
			switch(ot)
			{
				case OperandType.O2:	//˫Ŀ
					b=(double)_opnd.Pop();
					a=(double)_opnd.Pop();
					break;
				case OperandType.O1:	//Ŀ
					a=(double)_opnd.Pop();
					b=0.0;
					break;
				case OperandType.O0:	//Ŀ
					return 0.0;
			}
			//Let's calculate!!!
            double tmp = 0.0;
			switch(pt)
			{
				//
				case PhraseType.plus:
					return a+b;
				case PhraseType.minus:
					return a-b;
				case PhraseType.mutiple:
					return a*b;
				case PhraseType.divide:
					return a/b;
				case PhraseType.mod:
					return a%b;
                case PhraseType.negative:
                    return 0.0-a;
				case PhraseType.fact:
					return Factorial(a);
				//Ǻ
				case PhraseType.sin:
                    if (Mode == CalculationMode.Degree)
                    {
                        a = Utility.DegreesToRadians(a);
                    }
					return Math.Sin(a);
				case PhraseType.cos:
                    if (Mode == CalculationMode.Degree)
                    {
                        a = Utility.DegreesToRadians(a);
                    }
					return Math.Cos(a);
				case PhraseType.tg:
                    if (Mode == CalculationMode.Degree)
                    {
                        a = Utility.DegreesToRadians(a);
                    }
					return Math.Tan(a);
				case PhraseType.ctg:
                    if (Mode == CalculationMode.Degree)
                    {
                        a = Utility.DegreesToRadians(a);
                    }
					return 1.0/Math.Tan(a);
				case PhraseType.acos:
                    tmp=Math.Acos(a);
                    if(Mode==CalculationMode.Degree)
                    {
                        tmp=Utility.RadiansToDegrees(tmp);
                    }
                    return tmp;
				case PhraseType.asin:
                    tmp = Math.Asin(a);
                    if (Mode == CalculationMode.Degree)
                    {
                        tmp = Utility.RadiansToDegrees(tmp);
                    }
                    return tmp;
				case PhraseType.atg:
                    tmp = Math.Atan(a);
                    if (Mode == CalculationMode.Degree)
                    {
                        tmp = Utility.RadiansToDegrees(tmp);
                    }
                    return tmp;
				case PhraseType.actg:
                    tmp = Math.Atan(1.0/a);
                    if (Mode == CalculationMode.Degree)
                    {
                        tmp = Utility.RadiansToDegrees(tmp);
                    }
                    return tmp;
				//˷
				case PhraseType.pow:
					return Math.Pow(a,b);
				case PhraseType.sbrt:
					return Math.Sqrt(a);
				case PhraseType.cbrt:
					return Math.Pow(a,1.0/3.0);
				//log series
				case PhraseType.ln:
					return Math.Log(a,ConstantTable.EXP);
				case PhraseType.log:
					return Math.Log(b,a);		//a log b
				case PhraseType.lg:
					return Math.Log10(a);
				default:
					return 0.0;
			}
		}
		/// <summary>
		/// Run the calculation
		/// </summary>
		public void Run()
		{
			//еĴѹջ
			int i=0;
			_optr.Clear();
			_optr.Push(PhraseType.sharp);	//#Ϊջ־
			
			_opnd.Clear();

			while(i<_ps.Length)
			{
				PhraseType temp_pt=_ps.PhraseTypeResult[i];
				if(temp_pt==PhraseType.number)		//
					_opnd.Push(_ps.GetNumberValue(i));
				else if(temp_pt==PhraseType.e)		//e
					_opnd.Push(ConstantTable.EXP);
				else if(temp_pt==PhraseType.pi)		//pi
					_opnd.Push(ConstantTable.PI);
				else if(temp_pt==PhraseType.ans)	//ans
					_opnd.Push(ConstantTable.ANS);
				else if(temp_pt==PhraseType.ax||temp_pt==PhraseType.bx||temp_pt==PhraseType.cx
					||temp_pt==PhraseType.dx||temp_pt==PhraseType.ex||temp_pt==PhraseType.fx)
					_opnd.Push(ConstantTable.GetFrom(temp_pt.ToString()));
				else	//
				{
					//
					if((PhraseType)_optr.Peek()==PhraseType.sharp&&temp_pt==PhraseType.sharp)
						break;

					PriorityCmpType temp_pct=(PriorityCmpType)Operator.OperatorCmp2((PhraseType)_optr.Peek(),temp_pt);
					if(temp_pct==PriorityCmpType.Higher)
					{
						do
						{
							//ǰһ
							if(Operator.OperandCount((PhraseType)_optr.Peek())!=OperandType.O0)	//Ŀ
								_opnd.Push(Calculate((PhraseType)_optr.Pop()));

						}while((PriorityCmpType)Operator.OperatorCmp2((PhraseType)_optr.Peek(),temp_pt)==PriorityCmpType.Higher);
						//PhraseTypeȼʱ
						if((PriorityCmpType)Operator.OperatorCmp2((PhraseType)_optr.Peek(),temp_pt)==PriorityCmpType.Equal)
						{
							_optr.Pop();	//׳ȵprePhraseType
						}
						else
						{
							_optr.Push(temp_pt);
						}
					}
					else if(temp_pct==PriorityCmpType.Lower)
					{
						_optr.Push(temp_pt);
					}
					else if(temp_pct==PriorityCmpType.Equal)
					{
						_optr.Pop();
					}
				}
				i++;
			}
			//洢ans
			ConstantTable.ANS=(double)_opnd.Peek();
		}
		/// <summary>
		/// Result
		/// </summary>
		public string ANS
		{
			get{
				if(ConstantTable.ANS.CompareTo(double.NaN)==0		//none-number
					||ConstantTable.ANS.CompareTo(double.PositiveInfinity)==0	//positive infinity
					||ConstantTable.ANS.CompareTo(double.NegativeInfinity)==0)	//negitive infinity
					return "ʾΧ";
				else
					return ConstantTable.ANS.ToString();
			}
		}
	}
}
